@@ -25,6 +25,8 @@ import {
     STANDARD_HTTPS_PORT,
     STANDARD_WEB_PORT,
     SPECIAL_FILTER_ID,
+    THEMES,
+    LOCAL_STORAGE_THEME_KEY,
 } from './constants';
 
 /**
@@ -388,6 +390,12 @@ export const toggleAllServices = (services, change, isSelected) => {
     services.forEach((service) => change(`blocked_services.${service.id}`, isSelected));
 };
 
+export const msToSeconds = (milliseconds) => Math.floor(milliseconds / 1000);
+
+export const msToMinutes = (milliseconds) => Math.floor(milliseconds / 1000 / 60);
+
+export const msToHours = (milliseconds) => Math.floor(milliseconds / 1000 / 60 / 60);
+
 export const secondsToMilliseconds = (seconds) => {
     if (seconds) {
         return seconds * 1000;
@@ -396,6 +404,8 @@ export const secondsToMilliseconds = (seconds) => {
     return seconds;
 };
 
+export const msToDays = (milliseconds) => Math.floor(milliseconds / 1000 / 60 / 60 / 24);
+
 export const normalizeRulesTextarea = (text) => text?.replace(/^\n/g, '')
     .replace(/\n\s*\n/g, '\n');
 
@@ -670,13 +680,61 @@ export const setHtmlLangAttr = (language) => {
     window.document.documentElement.lang = language;
 };
 
+/**
+ * Set local storage field
+ *
+ * @param {string} key
+ * @param {string} value
+ */
+
+export const setStorageItem = (key, value) => {
+    if (window.localStorage) {
+        window.localStorage.setItem(key, value);
+    }
+};
+
+/**
+ * Get local storage field
+ *
+ * @param {string} key
+ */
+
+export const getStorageItem = (key) => (window.localStorage
+    ? window.localStorage.getItem(key)
+    : null);
+
+/**
+ * Set local storage theme field
+ *
+ * @param {string} theme
+ */
+
+export const setTheme = (theme) => {
+    setStorageItem(LOCAL_STORAGE_THEME_KEY, theme);
+};
+
+/**
+ * Get local storage theme field
+ *
+ * @returns {string}
+ */
+
+export const getTheme = () => getStorageItem(LOCAL_STORAGE_THEME_KEY) || THEMES.light;
+
 /**
  * Sets UI theme.
  *
  * @param theme
  */
 export const setUITheme = (theme) => {
-    document.body.dataset.theme = theme;
+    let currentTheme = theme || getTheme();
+
+    if (currentTheme === THEMES.auto) {
+        const prefersDark = window.matchMedia && window.matchMedia('(prefers-color-scheme: dark)').matches;
+        currentTheme = prefersDark ? THEMES.dark : THEMES.light;
+    }
+    setTheme(currentTheme);
+    document.body.dataset.theme = currentTheme;
 };
 
 /**
