@@ -15,13 +15,6 @@ import (
 )
 
 const (
-	// leaseExpireStatic is used to define the Expiry field for static
-	// leases.
-	//
-	// TODO(e.burkov): Remove it when static leases determining mechanism
-	// will be improved.
-	leaseExpireStatic = 1
-
 	// DefaultDHCPLeaseTTL is the default time-to-live for leases.
 	DefaultDHCPLeaseTTL = uint32(timeutil.Day / time.Second)
 
@@ -35,19 +28,25 @@ const (
 	defaultBackoff     time.Duration = 500 * time.Millisecond
 )
 
-// Lease contains the necessary information about a DHCP lease
+// Lease contains the necessary information about a DHCP lease.  It's used in
+// various places.  So don't change it without good reason.
 type Lease struct {
-	// Expiry is the expiration time of the lease.  The unix timestamp value
-	// of 1 means that this is a static lease.
+	// Expiry is the expiration time of the lease.
 	Expiry time.Time `json:"expires"`
 
-	Hostname string           `json:"hostname"`
-	HWAddr   net.HardwareAddr `json:"mac"`
+	// Hostname of the client.
+	Hostname string `json:"hostname"`
+
+	// HWAddr is the physical hardware address (MAC address).
+	HWAddr net.HardwareAddr `json:"mac"`
 
 	// IP is the IP address leased to the client.
 	//
-	// TODO(a.garipov): Migrate leases.db and use netip.Addr.
-	IP net.IP `json:"ip"`
+	// TODO(a.garipov): Migrate leases.db.
+	IP netip.Addr `json:"ip"`
+
+	// IsStatic defines if the lease is static.
+	IsStatic bool `json:"static"`
 }
 
 // Clone returns a deep copy of l.
@@ -60,7 +59,8 @@ func (l *Lease) Clone() (clone *Lease) {
 		Expiry:   l.Expiry,
 		Hostname: l.Hostname,
 		HWAddr:   slices.Clone(l.HWAddr),
-		IP:       slices.Clone(l.IP),
+		IP:       l.IP,
+		IsStatic: l.IsStatic,
 	}
 }
 
@@ -81,17 +81,10 @@ func (l *Lease) IsBlocklisted() (ok bool) {
 	return true
 }
 
-// IsStatic returns true if the lease is static.
-//
-// TODO(a.garipov): Just make it a boolean field.
-func (l *Lease) IsStatic() (ok bool) {
-	return l != nil && l.Expiry.Unix() == leaseExpireStatic
-}
-
 // MarshalJSON implements the json.Marshaler interface for Lease.
 func (l Lease) MarshalJSON() ([]byte, error) {
 	var expiryStr string
-	if !l.IsStatic() {
+	if !l.IsStatic {
 		// The front-end is waiting for RFC 3999 format of the time
 		// value.  It also shouldn't got an Expiry field for static
 		// leases.
@@ -238,7 +231,7 @@ func Create(conf *ServerConfig) (s *server, err error) {
 
 			LocalDomainName: conf.LocalDomainName,
 
-			DBFilePath: filepath.Join(conf.WorkDir, dbFilename),
+			dbFilePath: filepath.Join(conf.DataDir, dataFilename),
 		},
 	}
 
@@ -279,6 +272,13 @@ func Create(conf *ServerConfig) (s *server, err error) {
 		return nil, fmt.Errorf("neither dhcpv4 nor dhcpv6 srv is configured")
 	}
 
+	// Migrate leases db if needed.
+	err = migrateDB(conf)
+	if err != nil {
+		// Don't wrap the error since it's informative enough as is.
+		return nil, err
+	}
+
 	// Don't delay database loading until the DHCP server is started,
 	// because we need static leases functionality available beforehand.
 	err = s.dbLoad()
